@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2013, Peter Haag
+ *  Copyright (c) 2013-2020, Peter Haag
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -27,16 +26,8 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author$
- *
- *  $Id$
- *
- *  $LastChangedRevision$
- *  
  */
 
-/* $Id: pcapd.c 2778 2012-03-19 09:23:26Z roethlis $ */
-
 #include "config.h"
 
 #ifdef HAVE_FEATURES_H
@@ -62,6 +53,8 @@
 #include <errno.h>
 #include <signal.h>
 #include <string.h>
+#include <assert.h>
+// #include <mcheck.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -90,38 +83,22 @@
 #include <pcap.h>
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
-#include "nf_common.h"
+#include "expire.h"
 #include "nfnet.h"
 #include "flist.h"
 #include "nfstatfile.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
-#include "rbtree.h"
 #include "ipfrag.h"
-
-#ifdef HAVE_FTS_H
-#   include <fts.h>
-#else
-#   include "fts_compat.h"
-#define fts_children fts_children_compat
-#define fts_close fts_close_compat
-#define fts_open  fts_open_compat
-#define fts_read  fts_read_compat
-#define fts_set   fts_set_compat
-#endif
-
-#include "expire.h"
-
 #include "flowtree.h"
 #include "netflow_pcap.h"
 #include "pcaproc.h"
 
 #define TIME_WINDOW     300
-#define SNAPLEN         200
 #define PROMISC         1
 #define TIMEOUT         500
 #define FILTER          ""
@@ -131,11 +108,7 @@
 #define DLT_LINUX_SLL   113
 #endif
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
+#define EXPIREINTERVALL 10
 
 int verbose = 0;
 
@@ -146,7 +119,7 @@ int verbose = 0;
 
 static const char *nfdump_version = VERSION;
 
-static int launcher_alive, periodic_trigger, launcher_pid;
+static int launcher_pid;
 static pthread_mutex_t  m_done  = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t terminate = PTHREAD_COND_INITIALIZER;
 static pthread_key_t buffer_key;
@@ -173,6 +146,7 @@ typedef struct p_pcap_flush_thread_args_s {
 	// arguments
 	int		subdir_index;
 	char	*pcap_datadir;
+	char	*time_extension;
 	pcap_dev_t *pcap_dev; 
 	pcapfile_t *pcapfile;
 } p_pcap_flush_thread_args_t;
@@ -192,15 +166,15 @@ typedef struct p_packet_thread_args_s {
 	time_t	t_win;
 	int		subdir_index;
 	char	*pcap_datadir;
+	char	*time_extension;
 	int		live;
 } p_packet_thread_args_t;
 
 typedef struct p_flow_thread_args_s {
 	// common thread info struct
 	pthread_t tid;
-	int		  done;
-	int		  exit;
-
+	int	done;
+	int	exit;
 	// the parent
 	pthread_t parent;
 
@@ -208,8 +182,10 @@ typedef struct p_flow_thread_args_s {
 	NodeList_t *NodeList;		// pop new nodes from this list
 	FlowSource_t *fs;
 	time_t	t_win;
+	char	*time_extension;
 	int		subdir_index;
 	int		compress;
+	int		live;
 } p_flow_thread_args_t;
 
 /*
@@ -248,17 +224,20 @@ static void usage(char *name) {
 					"-h\t\tthis text you see right here\n"
 					"-u userid\tChange user to username\n"
 					"-g groupid\tChange group to groupname\n"
-					"-i device\tspecify a device\n"
-					"-r pcapfile\tspecify a file to read from\n"
-					"-B cache buckets\tset the number of cache buckets. (default 1048576)\n"
-					"-s snaplen\tset the snapshot length\n"
+					"-i interface\tread packets from interface\n"
+					"-r pcapfile\tread packets from file\n"
+					"-B num\tset the node cache size. (default 524288)\n"
+					"-s snaplen\tset the snapshot length - default 1526\n"
+					"-e active,inactive\tset the active,inactive flow expire time (s) - default 300,60\n"
 					"-l flowdir \tset the flow output directory. (no default) \n"
-					"-l pcapdir \tset the pcapdir directory. (optional) \n"
+					"-p pcapdir \tset the pcapdir directory. (optional) \n"
 					"-S subdir\tSub directory format. see nfcapd(1) for format\n"
 					"-I Ident\tset the ident string for stat file. (default 'none')\n"
 					"-P pidfile\tset the PID file\n"
 					"-t time frame\tset the time window to rotate pcap/nfcapd file\n"
-					"-z\t\tCompress flows in output file.\n"
+					"-z\t\tLZO compress flows in output file.\n"
+					"-y\t\tLZ4 compress flows in output file.\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
 					"-E\t\tPrint extended format of netflow data. for debugging purpose only.\n"
 					"-T\t\tInclude extension tags in records.\n"
 					"-D\t\tdetach from terminal (daemonize)\n"
@@ -269,13 +248,12 @@ static void Interrupt_handler(int sig) {
 pthread_t tid		 = pthread_self();
 thread_info_t	*thread_info;
 
-	LogInfo("[%lu] Signal handler: %i", (long unsigned)tid, sig);
 	thread_info = (thread_info_t *)pthread_getspecific(buffer_key);
 	if ( !thread_info ) {
-		LogError("[%lu] Interrupt_handler() failed to get thread specific data block\n", (long unsigned)tid);
+		LogError("[%lu] Interrupt_handler() failed to get thread specific data block", (long unsigned)tid);
 	} else {
 		if ( thread_info->tid != tid ) {
-			LogError("[%lu] Interrupt_handler() missmatch tid in thread_info\n", (long unsigned)tid);
+			LogError("[%lu] Interrupt_handler() missmatch tid in thread_info", (long unsigned)tid);
 		} else {
 			thread_info->done = 1;
 		}
@@ -394,8 +372,9 @@ int		err;
 } // End of SetPriv
 
 static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen) {
-pcap_t 		*handle;
-pcap_dev_t	*pcap_dev;
+pcap_t 		*handle    = NULL;
+pcap_dev_t	*pcap_dev  = NULL;
+pcap_if_t	*alldevsp = NULL;
 char errbuf[PCAP_ERRBUF_SIZE];
 bpf_u_int32 mask;		/* Our netmask */
 bpf_u_int32 net;		/* Our IP */
@@ -405,11 +384,17 @@ uint32_t	linkoffset, linktype;
 	dbg_printf("Enter function: %s\n", __FUNCTION__);
 
 	if (device == NULL) {
-		device = pcap_lookupdev(errbuf);
-		if (device == NULL) {
-			LogError("Couldn't find default device: %s", errbuf);
+		if ( pcap_findalldevs(&alldevsp, errbuf) == -1 ) {
+			LogError("pcap_findalldevs() error: %s in %s line %d", 
+				errbuf, __FILE__, __LINE__);
 			return NULL;
 		}
+		if ( alldevsp == NULL ) {
+			LogError("Couldn't find default device");
+			return NULL;
+		}
+		device = alldevsp[0].name;
+		LogInfo("Listen on %s", device);
 	}
 
 	/* Find the properties for the device */
@@ -422,7 +407,6 @@ uint32_t	linkoffset, linktype;
 	/*
 	 *  Open the packet capturing device with the following values:
 	 *
-	 *  SNAPLEN: User defined or 200 bytes
 	 *  PROMISC: on
 	 *  The interface needs to be in promiscuous mode to capture all
 	 *  network traffic on the localnet.
@@ -612,13 +596,14 @@ static void SignalThreadTerminate(thread_info_t *thread_info, pthread_cond_t *th
 __attribute__((noreturn)) static void *p_flow_thread(void *thread_data) {
 // argument dispatching
 p_flow_thread_args_t *args = (p_flow_thread_args_t *)thread_data;
-time_t t_win				 = args->t_win;
-int subdir_index			 = args->subdir_index;
-int compress			 	 = args->compress;
-FlowSource_t *fs			 = args->fs;
-
-// locals
-time_t t_start, t_clock, t_udp_flush;
+time_t t_win		 = args->t_win;
+int subdir_index	 = args->subdir_index;
+int compress	 	 = args->compress;
+int live		 	 = args->live;
+FlowSource_t *fs	 = args->fs;
+char *time_extension = args->time_extension;
+static time_t lastExpire = 0;
+time_t t_start, t_clock;
 int err, done;
 
 	done 	   = 0;
@@ -650,7 +635,6 @@ int err, done;
    		pthread_kill(args->parent, SIGUSR1);
 		pthread_exit((void *)args);
 	}
-	fs->xstat = NULL;
 
 	// init vars
 	fs->bad_packets		= 0;
@@ -659,7 +643,6 @@ int err, done;
 
 	t_start = 0;
 	t_clock = 0;
-	t_udp_flush = 0;
 	while ( 1 ) {
 		struct FlowNode	*Node;
 
@@ -673,7 +656,7 @@ int err, done;
 		}
 
 		if ( t_start == 0 ) {
-			t_udp_flush = t_start = t_clock - (t_clock % t_win);
+			t_start = t_clock - (t_clock % t_win);
 		}
 
 		if (((t_clock - t_start) >= t_win) || done) { /* rotate file */
@@ -682,13 +665,19 @@ int err, done;
 			char FullName[MAXPATHLEN];
 			char netflowFname[128];
 			char error[256];
-			char *subdir;
+			char *subdir, fmt[24];
+			uint32_t NumFlows;
 
 			// flush all flows to disk
-			DumpNodeStat();
-			uint32_t NumFlows  = Flush_FlowTree(fs);
+			DumpNodeStat(args->NodeList);
+			if (done)
+				NumFlows = Flush_FlowTree(fs);
+			else
+				NumFlows = Expire_FlowTree(fs, t_clock);
 
 			when = localtime(&t_start);
+			strftime(fmt, sizeof(fmt), time_extension, when);
+
 			nffile = fs->nffile;
 
 			// prepare sub dir hierarchy
@@ -700,17 +689,14 @@ int err, done;
 				
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(netflowFname, 127, "nfcapd.%i%02i%02i%02i%02i",
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(netflowFname, 127, "nfcapd.%s", fmt);
 				} else {
-					snprintf(netflowFname, 127, "%s/nfcapd.%i%02i%02i%02i%02i", subdir,
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(netflowFname, 127, "%s/nfcapd.%s", subdir, fmt);
 				}
 
 			} else {
 				subdir = NULL;
-				snprintf(netflowFname, 127, "nfcapd.%i%02i%02i%02i%02i",
-					when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+				snprintf(netflowFname, 127, "nfcapd.%s", fmt);
 			}
 			netflowFname[127] = '\0';
 	
@@ -769,9 +755,9 @@ int err, done;
 				UpdateBooks(fs->bookkeeper, t_start, 512*fstat.st_blocks);
 			}
 
-			LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Max Flows: %u", 
+			LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Max Flows: %u, Fragments: %u", 
 				fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
-				(unsigned long long)nffile->stat_record->numbytes, NumFlows);
+				(unsigned long long)nffile->stat_record->numbytes, NumFlows, IPFragEntries());
 
 			// reset stats
 			fs->bad_packets = 0;
@@ -796,14 +782,21 @@ int err, done;
 			}
 		}
 
-		if (((t_clock - t_udp_flush) >= 10) || !done) { /* flush inactive UDP list */
-			UDPexpire(fs, t_clock - 10 );
-			t_udp_flush = t_clock;
-		}
-
-		if ( Node->fin != SIGNAL_NODE )
-			// Process the Node
-			ProcessFlowNode(fs, Node);
+		time_t when;
+		if ( Node ) {
+			if ( Node->fin != SIGNAL_NODE ) {
+				// Process the Node
+				ProcessFlowNode(fs, Node);
+			}
+			when = Node->t_last.tv_sec;
+		} else {
+			when = time(NULL);
+		} 
+		if ( (when - lastExpire) > EXPIREINTERVALL ) {
+			Expire_FlowTree(fs, when);
+			lastExpire = when;
+		} 
+		CacheCheck(fs, when, live);
 
 	}
 
@@ -825,6 +818,7 @@ p_pcap_flush_thread_args_t *args = (p_pcap_flush_thread_args_t *)thread_data;
 char *pcap_datadir	 = args->pcap_datadir;
 pcap_dev_t *pcap_dev = args->pcap_dev;
 pcapfile_t *pcapfile = args->pcapfile;
+char *time_extension = args->time_extension;
 char pcap_dumpfile[MAXPATHLEN];
 int err;
 int runs = 0;
@@ -889,11 +883,13 @@ int runs = 0;
 			char FullName[MAXPATHLEN];
 			char pcapFname[128];
 			char error[256];
-			char *subdir;
+			char *subdir, fmt[24];
 			int err;
 
 			dbg_printf("Flush rotate file\n");
 			when = localtime(&pcapfile->t_CloseRename);
+			strftime(fmt, sizeof(fmt), time_extension, when);
+
 			pcapfile->t_CloseRename = 0;
 
 			// prepare sub dir hierarchy
@@ -905,17 +901,14 @@ int runs = 0;
 				
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(pcapFname, 127, "pcapd.%i%02i%02i%02i%02i",
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(pcapFname, 127, "pcapd.%s", fmt);
 				} else {
-					snprintf(pcapFname, 127, "%s/pcapd.%i%02i%02i%02i%02i", subdir,
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(pcapFname, 127, "%s/pcapd.%s", subdir, fmt);
 				}
 
 			} else {
 				subdir = NULL;
-				snprintf(pcapFname, 127, "pcapd.%i%02i%02i%02i%02i",
-					when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+				snprintf(pcapFname, 127, "pcapd.%s", fmt);
 			}
 			pcapFname[127] 	  = '\0';
 	
@@ -971,6 +964,7 @@ p_packet_thread_args_t *args = (p_packet_thread_args_t *)thread_data;
 pcap_dev_t *pcap_dev = args->pcap_dev;
 time_t t_win		 = args->t_win;
 char *pcap_datadir 	 = args->pcap_datadir;
+char *time_extension = args->time_extension;
 int subdir_index	 = args->subdir_index;
 int live		 	 = args->live;
 // locals
@@ -1006,13 +1000,14 @@ int err;
 			/* NOTREACHED */
 		}
 
-		p_flush_thread_args.done 	 	 = 0;
-		p_flush_thread_args.exit 	 	 = 0;
-		p_flush_thread_args.parent 	 	 = args->tid;
-		p_flush_thread_args.pcap_dev 	 = args->pcap_dev;
-		p_flush_thread_args.subdir_index = subdir_index;
-		p_flush_thread_args.pcap_datadir = pcap_datadir;
-		p_flush_thread_args.pcapfile 	 = pcapfile;
+		p_flush_thread_args.done		   = 0;
+		p_flush_thread_args.exit		   = 0;
+		p_flush_thread_args.parent		   = args->tid;
+		p_flush_thread_args.pcap_dev	   = args->pcap_dev;
+		p_flush_thread_args.subdir_index   = subdir_index;
+		p_flush_thread_args.pcap_datadir   = pcap_datadir;
+		p_flush_thread_args.time_extension = time_extension;
+		p_flush_thread_args.pcapfile	   = pcapfile;
 
 		err = pthread_create(&p_flush_thread_args.tid, NULL, p_pcap_flush_thread, (void *)&p_flush_thread_args);
 		if ( err ) {
@@ -1072,13 +1067,23 @@ int err;
 							Node->t_last  = tv;
 							Node->fin  	  = SIGNAL_NODE;
 							Push_Node(args->NodeList, Node);
-							if ( pcap_datadir )
-							// keep the packet
+							if ( pcap_datadir ) {
+								// keep the packet
 								RotateFile(pcapfile, t_start, live);
+							}
 							LogInfo("Packet processing stats: Total: %u, Skipped: %u, Unknown: %u, Short snaplen: %u", 
 								pcap_dev->proc_stat.packets, pcap_dev->proc_stat.skipped, 
 								pcap_dev->proc_stat.unknown, pcap_dev->proc_stat.short_snap);
 						}
+						if ( live ) {
+							struct pcap_stat p_stat;
+							if( pcap_stats(pcap_dev->handle, &p_stat) < 0) {
+								LogInfo("pcap_stats() failed: %s", pcap_geterr(pcapfile->p));
+							} else {
+								LogInfo("Dropped: %u, dropped by interface: %u ",
+									p_stat.ps_drop, p_stat.ps_ifdrop );
+							}
+						}
 						t_start = t_clock - (t_clock % t_win);
 						memset((void *)&(pcap_dev->proc_stat), 0, sizeof(proc_stat_t));
 					} 
@@ -1187,16 +1192,18 @@ sigset_t			signal_set;
 struct sigaction	sa;
 int c, snaplen, err, do_daemonize;
 int subdir_index, compress, expire, cache_size;
+int active, inactive;
 FlowSource_t	*fs;
 dirstat_t 		*dirstat;
 time_t 			t_win;
 char 			*device, *pcapfile, *filter, *datadir, *pcap_datadir, *extension_tags, pidfile[MAXPATHLEN], pidstr[32];
 char			*Ident, *userid, *groupid;
+char			*time_extension;
 pcap_dev_t 		*pcap_dev;
 p_packet_thread_args_t *p_packet_thread_args;
 p_flow_thread_args_t *p_flow_thread_args;
 
-	snaplen			= 100;
+	snaplen			= 1526;
 	do_daemonize	= 0;
 	launcher_pid	= 0;
 	device			= NULL;
@@ -1210,12 +1217,15 @@ p_flow_thread_args_t *p_flow_thread_args;
 	Ident			= "none";
 	fs				= NULL;
 	extension_tags	= DefaultExtensions;
+	time_extension	= "%Y%m%d%H%M";
 	subdir_index	= 0;
 	compress		= NOT_COMPRESSED;
 	verbose			= 0;
 	expire			= 0;
 	cache_size		= 0;
-	while ((c = getopt(argc, argv, "B:DEI:g:hi:j:r:s:l:p:P:t:u:S:T:e:Vz")) != EOF) {
+	active			= 0;
+	inactive		= 0;
+	while ((c = getopt(argc, argv, "B:DEI:e:g:hi:j:r:s:l:p:P:t:u:S:T:Vyz")) != EOF) {
 		switch (c) {
 			struct stat fstat;
 			case 'h':
@@ -1279,15 +1289,35 @@ p_flow_thread_args_t *p_flow_thread_args;
 					exit(EXIT_FAILURE);
 				}
 				break;
+			case 'e': {
+				if ( strlen(optarg) > 16 ) {
+					LogError("ERROR:, size timeout values too big");
+					exit(EXIT_FAILURE);
+				}
+				char *s = strdup(optarg);
+				char *sep = strchr(s, ',');
+				if ( !sep ) {
+					LogError("ERROR:, timeout values format error");
+					exit(EXIT_FAILURE);
+				}
+				*sep = '\0';
+				sep++;
+				active   = atoi(s);
+				inactive = atoi(sep);
+				if (snaplen < 14 + 20 + 20) { // ethernet, IP , TCP, no payload
+					LogError("ERROR:, snaplen < sizeof IPv4 - Need 54 bytes for TCP/IPv4");
+					exit(EXIT_FAILURE);
+				}
+				} break;
 			case 't':
 				t_win = atoi(optarg);
-				if (t_win < 60) {
-					LogError("WARNING, very small time frame (< 60)!");
-				}
-				if (t_win <= 0) {
-					LogError("ERROR: time frame too small (<= 0)");
+				if (t_win < 2) {
+					LogError("time interval <= 2s not allowed");
 					exit(EXIT_FAILURE);
 				}
+				if (t_win < 60) {
+					time_extension	= "%Y%m%d%H%M%S";
+				}
 				break;
 			case 'j':
 				if ( compress ) {
@@ -1296,6 +1326,13 @@ p_flow_thread_args_t *p_flow_thread_args;
 				}
 				compress = BZ2_COMPRESSED;
 				break;
+			case 'y':
+				if ( compress ) {
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+					exit(255);
+				}
+				compress = LZ4_COMPRESSED;
+				break;
 			case 'z':
 				if ( compress ) {
 					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
@@ -1313,7 +1350,12 @@ p_flow_thread_args_t *p_flow_thread_args;
 						exit(255);
 					}
 					tmp[MAXPATHLEN-1] = 0;
-					snprintf(pidfile, MAXPATHLEN - 1 - strlen(tmp), "%s/%s", tmp, optarg);
+					if ( (strlen(tmp) + strlen(optarg) + 3) < MAXPATHLEN ) {
+						snprintf(pidfile, MAXPATHLEN - 3 - strlen(tmp), "%s/%s", tmp, optarg);
+					} else {
+						fprintf(stderr, "pidfile MAXPATHLEN error:\n");
+						exit(255);
+					}
 				}
 				// pidfile now absolute path
 				pidfile[MAXPATHLEN-1] = 0;
@@ -1366,8 +1408,7 @@ p_flow_thread_args_t *p_flow_thread_args;
 		exit(EXIT_FAILURE);
 	}
 
-	
-	if ( !Init_FlowTree(cache_size)) {
+	if ( !Init_FlowTree(cache_size, active, inactive)) {
 		LogError("Init_FlowTree() failed.");
 		exit(EXIT_FAILURE);
 	}
@@ -1391,7 +1432,7 @@ p_flow_thread_args_t *p_flow_thread_args;
 		exit(255);
 	}
 
-	if ( do_daemonize && !InitLog(argv[0], SYSLOG_FACILITY)) {
+	if ( !InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose) ) {
 		pcap_close(pcap_dev->handle);
 		exit(255);
 	}
@@ -1507,14 +1548,17 @@ p_flow_thread_args_t *p_flow_thread_args;
 			__FILE__, __LINE__, strerror(errno) );
 		exit(255);
 	}	
-	p_flow_thread_args->fs 	   	   = fs;
-	p_flow_thread_args->t_win 	   = t_win;
-	p_flow_thread_args->compress   = compress;
-	p_flow_thread_args->subdir_index = subdir_index;
-	p_flow_thread_args->parent	   = pthread_self();
-	p_flow_thread_args->NodeList   = NewNodeList();
+	p_flow_thread_args->fs			   = fs;
+	p_flow_thread_args->t_win		   = t_win;
+	p_flow_thread_args->compress	   = compress;
+	p_flow_thread_args->live		   = device != NULL;
+	p_flow_thread_args->subdir_index   = subdir_index;
+	p_flow_thread_args->parent		   = pthread_self();
+	p_flow_thread_args->NodeList	   = NewNodeList();
+	p_flow_thread_args->time_extension = time_extension;
 
 	err = 0;
+
 	err = pthread_create(&p_flow_thread_args->tid, NULL, p_flow_thread, (void *)p_flow_thread_args);
 	if ( err ) {
 		LogError("pthread_create() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
@@ -1529,13 +1573,14 @@ p_flow_thread_args_t *p_flow_thread_args;
 			__FILE__, __LINE__, strerror(errno) );
 		exit(255);
 	}	
-	p_packet_thread_args->pcap_dev 	   = pcap_dev;
-	p_packet_thread_args->t_win 	   = t_win;
-	p_packet_thread_args->subdir_index = subdir_index;
-	p_packet_thread_args->pcap_datadir = pcap_datadir;
-	p_packet_thread_args->live 	   	   = device != NULL;
-	p_packet_thread_args->parent	   = pthread_self();
-	p_packet_thread_args->NodeList	   = p_flow_thread_args->NodeList;
+	p_packet_thread_args->pcap_dev		 = pcap_dev;
+	p_packet_thread_args->t_win			 = t_win;
+	p_packet_thread_args->subdir_index	 = subdir_index;
+	p_packet_thread_args->pcap_datadir	 = pcap_datadir;
+	p_packet_thread_args->live			 = device != NULL;
+	p_packet_thread_args->parent		 = pthread_self();
+	p_packet_thread_args->NodeList		 = p_flow_thread_args->NodeList;
+	p_packet_thread_args->time_extension = p_flow_thread_args->time_extension;
 
 	err = pthread_create(&p_packet_thread_args->tid, NULL, p_packet_thread, (void *)p_packet_thread_args);
 	if ( err ) {
@@ -1553,6 +1598,8 @@ p_flow_thread_args_t *p_flow_thread_args;
 	dbg_printf("Signal flow thread to terminate\n");
 	SignalThreadTerminate((thread_info_t *)p_flow_thread_args, &p_packet_thread_args->NodeList->c_list);
 
+	IPFragTree_free();
+
 	// free arg list
 	free((void *)p_packet_thread_args);
 	free((void *)p_flow_thread_args);
