@@ -70,7 +70,7 @@ int cmp(element *a, element *b) {
  *     list = listsort(mylist);
  */
 hash_table_row_hdr_t * list_sort(hash_table_row_hdr_t  *list, list_sort_callback_t callback, void *opts) {
-    hash_table_row_hdr_t *p, *q, *e, *tail, *oldhead;
+    hash_table_row_hdr_t *p, *q, *e, *tail;
     int insize, nmerges, psize, qsize, i;
 
     /*
@@ -84,7 +84,6 @@ hash_table_row_hdr_t * list_sort(hash_table_row_hdr_t  *list, list_sort_callback
 
     while (1) {
         p = list;
-	oldhead = list;		       /* only used for circular linkage */
         list = NULL;
         tail = NULL;
 
